﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace TestProject
{
    /// <summary>
    /// DPI 인식 이미지
    /// </summary>
    public class DPIAwareImage : Image
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 최적 스케일
        /// </summary>
        private double bestScale;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DPIAwareImage()

        /// <summary>
        /// 생성자
        /// </summary>
        public DPIAwareImage() : base()
        {
            Initialized += Image_Initialized;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected
        //////////////////////////////////////////////////////////////////////////////// Function

        #region DPI 변경시 처리하기 - OnDpiChanged(oldDPI, newDPI)

        /// <summary>
        /// DPI 변경시 처리하기
        /// </summary>
        /// <param name="oldDPI">구 DPI</param>
        /// <param name="newDPI">신 DPI</param>
        protected override void OnDpiChanged(DpiScale oldDPI, DpiScale newDPI)
        {
            ScaleImage(newDPI);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 이미지 초기화시 처리하기 - Image_Initialized(sender, e)

        /// <summary>
        /// 이미지 초기화시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Image_Initialized(object sender, EventArgs e)
        {
            DpiScale dpiScale = VisualTreeHelper.GetDpi(sender as Visual);

            ScaleImage(dpiScale);
        }

        #endregion
        #region 비트맵 이미지 다운로드 완료시 처리하기 - bitmapImage_DownloadCompleted(sender, e)

        /// <summary>
        /// 비트맵 이미지 다운로드 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void bitmapImage_DownloadCompleted(object sender, EventArgs e)
        {
            BitmapImage bitmapImage = sender as BitmapImage;

            SetImageSize(bitmapImage);

            bitmapImage.DownloadCompleted -= bitmapImage_DownloadCompleted;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 이미지 크기 설정하기 - SetImageSize(bitmapImage)

        /// <summary>
        /// 이미지 크기 설정하기
        /// </summary>
        /// <param name="bitmapImage">비트맵 이미지</param>
        private void SetImageSize(BitmapImage bitmapImage)
        {
            SetCurrentValue(Image.WidthProperty , bitmapImage.PixelWidth  * 100 / this.bestScale);
            SetCurrentValue(Image.HeightProperty, bitmapImage.PixelHeight * 100 / this.bestScale);
        }

        #endregion
        #region 이미지 업데이트하기 - UpdateImage(image, uriString)

        /// <summary>
        /// 이미지 업데이트하기
        /// </summary>
        /// <param name="image">이미지</param>
        /// <param name="uriString">URI 문자열</param>
        private void UpdateImage(Image image, string uriString)
        {
            Uri uri = new Uri(uriString, UriKind.RelativeOrAbsolute);

            BitmapImage bitmapImage = new BitmapImage();
            
            bitmapImage.BeginInit();

            bitmapImage.UriSource = uri;

            bitmapImage.EndInit();

            image.Source = bitmapImage;

            if(!bitmapImage.IsDownloading)
            {
                SetImageSize(bitmapImage);
            }
            else
            {
                bitmapImage.DownloadCompleted += bitmapImage_DownloadCompleted;
            }
        }

        #endregion
        #region 이미지 스케일 적용하기 - ScaleImage(dpiScale)

        /// <summary>
        /// 이미지 스케일 적용하기
        /// </summary>
        /// <param name="dpiScale">DPI 스케일</param>
        private void ScaleImage(DpiScale dpiScale)
        {
            this.bestScale = ImageDPIHelper.GetBestScale(dpiScale.PixelsPerDip);

            string imageURL = ImageDPIHelper.GetDesiredImageURL(this);

            UpdateImage(this, imageURL);
        }

        #endregion
    }
}